"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const External_1 = tslib_1.__importDefault(require("../../../types/External"));
const values_1 = require("../../../values");
const errors = tslib_1.__importStar(require("../../../errors"));
const decodeSequence_1 = tslib_1.__importDefault(require("../../der/decoders/decodeSequence"));
function decodeExternal(value) {
    const components = (0, decodeSequence_1.default)(value);
    let i = 0;
    const directReference = ((components[i]?.tagNumber === values_1.ASN1UniversalType.objectIdentifier)
        && (components[i].tagClass === values_1.ASN1TagClass.universal))
        ? components[i++].objectIdentifier
        : undefined;
    const indirectReference = ((components[i]?.tagNumber === values_1.ASN1UniversalType.integer)
        && (components[i].tagClass === values_1.ASN1TagClass.universal))
        ? components[i++].integer
        : undefined;
    const dataValueDescriptor = ((components[i]?.tagNumber === values_1.ASN1UniversalType.objectDescriptor)
        && (components[i].tagClass === values_1.ASN1TagClass.universal))
        ? components[i++].objectDescriptor
        : undefined;
    if (!directReference && !indirectReference) {
        throw new errors.ASN1ConstructionError("EXTERNAL must have direct or indirect reference.");
    }
    const encodingElement = components[i];
    if (!encodingElement || encodingElement.tagClass !== values_1.ASN1TagClass.context) {
        throw new errors.ASN1ConstructionError("EXTERNAL missing 'encoding' component.");
    }
    const encoding = (() => {
        switch (encodingElement.tagNumber) {
            case (0): return encodingElement.inner;
            case (1): return encodingElement.octetString;
            case (2): return encodingElement.bitString;
            default: {
                throw new errors.ASN1UndefinedError("EXTERNAL does not know of an encoding option "
                    + `having tag number ${encodingElement.tagNumber}.`);
            }
        }
    })();
    return new External_1.default(directReference, indirectReference, dataValueDescriptor, encoding);
}
exports.default = decodeExternal;
//# sourceMappingURL=decodeExternal.js.map