"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const errors = tslib_1.__importStar(require("../../../errors"));
const bigint_1 = require("../../../utils/bigint");
function decodeInteger(value) {
    if (value.length === 0) {
        throw new errors.ASN1SizeError("INTEGER or ENUMERATED encoded on zero bytes");
    }
    if (value.length > 2
        && ((value[0] === 0xFF && value[1] >= 0b10000000)
            || (value[0] === 0x00 && value[1] < 0b10000000))) {
        const buf = Buffer.from(value.slice(0, 16));
        throw new errors.ASN1PaddingError("Unnecessary padding bytes on INTEGER or ENUMERATED. "
            + `First 16 bytes of the offending value were: 0x${buf.toString("hex")}`);
    }
    return (0, bigint_1.bufferToInteger)(value);
}
exports.default = decodeInteger;
//# sourceMappingURL=decodeInteger.js.map