"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ObjectIdentifier_1 = tslib_1.__importDefault(require("../../../types/ObjectIdentifier"));
const errors = tslib_1.__importStar(require("../../../errors"));
function decodeObjectIdentifier(value) {
    if (value.length === 0) {
        throw new errors.ASN1TruncationError("Encoded value was too short to be an OBJECT IDENTIFIER!");
    }
    if (value.length > 1 && value[value.length - 1] & 0b10000000) {
        throw new errors.ASN1TruncationError("OID was truncated.");
    }
    const nodes = [0, 0];
    if (value[0] >= 0x50) {
        nodes[0] = 2;
        nodes[1] = (value[0] - 0x50);
    }
    else if (value[0] >= 0x28) {
        nodes[0] = 1;
        nodes[1] = (value[0] - 0x28);
    }
    else {
        nodes[0] = 0;
        nodes[1] = value[0];
    }
    if (value.length === 1) {
        return new ObjectIdentifier_1.default(nodes);
    }
    let current_node = 0;
    for (let i = 1; i < value.length; i++) {
        const byte = value[i];
        if ((byte === 0x80) && (current_node === 0)) {
            throw new errors.ASN1PaddingError("Prohibited padding on OBJECT IDENTIFIER node.");
        }
        current_node <<= 7;
        current_node += (byte & 127);
        if ((byte & 128) === 0) {
            nodes.push(current_node);
            current_node = 0;
        }
    }
    return new ObjectIdentifier_1.default(nodes);
}
exports.default = decodeObjectIdentifier;
//# sourceMappingURL=decodeObjectIdentifier.js.map