"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const base256Length_1 = tslib_1.__importDefault(require("./base256Length"));
const getBitFromBase128_1 = tslib_1.__importDefault(require("./getBitFromBase128"));
const setBitInBase256_1 = tslib_1.__importDefault(require("./setBitInBase256"));
function decodeBase128(value) {
    if (value.length === 1) {
        return value[0] & 127;
    }
    else if (value.length === 2) {
        const b1 = (value[0] & 127) >>> 1;
        const b2 = (value[1] & 127) | ((value[0] & 1) << 7);
        return Buffer.from([b1, b2]).readUint16BE();
    }
    else if (value.length === 3) {
        const b1 = (value[0] & 127) >>> 1;
        const b2 = (value[1] & 127) | ((value[0] & 1) << 7);
        const b3 = (value[1] & 127) | ((value[0] & 1) << 7);
        return Buffer.from([0, b1, b2, b3]).readUint32BE();
    }
    const decodedBytes = new Uint8Array((0, base256Length_1.default)(value.length));
    for (let byte = 0; byte < value.length; byte++) {
        for (let bit = 0; bit < 8; bit++) {
            const bitIndex = ((byte << 3) + bit);
            const bitValue = (0, getBitFromBase128_1.default)(value, bitIndex);
            if (bitValue) {
                (0, setBitInBase256_1.default)(decodedBytes, bitIndex, true);
            }
        }
    }
    if (decodedBytes.length === 3) {
        const buf = Buffer.alloc(4);
        buf.set(decodedBytes, 1);
        return buf.readUint32BE();
    }
    if (decodedBytes[0] === 0x80) {
        return Buffer.from(decodedBytes.slice(1)).readUint32BE();
    }
    return Buffer.from(decodedBytes).readUint32BE();
}
exports.default = decodeBase128;
//# sourceMappingURL=decodeBase128.js.map